
; ---------------------------------------------------------------------------
; NewTestBed.asm
; ---------------------------------------------------------------------
; Use a Console Screen Format into an array of
; bytes, converts it into a CHAR_INFO structure, 
; and INVOKEs the API: WriteConsoleOutput()
; to display it all at once . 
; Check for Windows version and display it. 
; Check the CPU brand and display it
; Wait a key to terminate the program or to repeat the test.
; Only ESC and R/r are accepted.
;----------------------------------------------------------------------
; Input file: Precompiled DB generated by Bin2dbx
;----------------------------------------------------------------------
; Author: frktons @ MASM32 forum
; Date: 03/nov/2010. 
;---------------------------------------------------------------------- 
; In this version the sizes of the screen and the console are read 
; from the first 4 bytes, indicating RowInitial, RowFinal, ColInitial,
; ColFinal.
;----------------------------------------------------------------------
; Frank
;----------------------------------------------------------------------------


include \masm32\include\masm32rt.inc



.686
.mmx
.xmm

;----------------------------------------------------------------------------
; PROC PROTO
;----------------------------------------------------------------------------

DisplayFmt   PROTO :DWORD
ClearBuffer  PROTO :DWORD
GetWindowsOS PROTO :PTR BYTE
DisplayAt    PROTO x:DWORD, y:DWORD, StringBuffer:DWORD, len:DWORD
SetColorTo   PROTO ForeGroundColor:BYTE, BackGroundColor:BYTE
get_cpu_id   PROTO :DWORD

;----------------------------------------------------------------------------
; Numeric constants used by the program
;----------------------------------------------------------------------------

Zero          EQU 0                  ;initial position on the buffer
Four          EQU 4                  ;number of initial bytes to read

MaxRows       EQU 34                 ;Max Rows of Console
MaxCols       EQU 90                 ;Max Cols of Console

CharNumber    EQU MaxRows*MaxCols

MaxSize       EQU MaxRows*MaxCols*Four  ;Max buffer size

PRODUCT_PROFESSIONAL EQU 30h         ; defined in WinNT.h Windows SDK 7.0A

BrightWhite    EQU 15
BrightYellow   EQU 14
DarkBlue       EQU 1

EscKey         EQU 27
R_Key          EQU 82
r_Key          EQU 114
S_Key          EQU 83
s_Key          EQU 115
V_Key          EQU 86
v_Key          EQU 118


;----------------------------------------------------------------------------
; Initialized data
;----------------------------------------------------------------------------

.data


    ConTitle  db "  ----------     **  New Testbed General Screen File "   
              db "Display Tool  **      ---------- Ver. 0.20 beta", 0   



    AlgoToTest    db   "Conversion of a screen buffer from"
                  db   " DOS to Windows CHAR_INFO structure",0  

    R_Key_Pressed db   "Chosen to Repeat the test      ",0
    S_Key_Pressed db   "Chosen to Save the result      ",0 
    V_Key_Pressed db   "Chosen to View the saved result",0       

;----------------------------------------------------------------------------
; Include for the Screen to display - precompiled db
;----------------------------------------------------------------------------  

include NewTestBed.inc


    
          
;----------------------------------------------------------------------------
; Uninitialized data
;----------------------------------------------------------------------------         

.data?

    ConsoleScreen CHAR_INFO CharNumber dup (<>)

    SecurityArea  db   16 dup (?)

    OddNumber     dd   ?

    ChosenColor   dw   ?
    ActualColor   dw   ? 

    Lenght        dd   ?
        
  

    R1            dw   ?
    R2            dw   ?
    C1            dw   ?
    C2            dw   ?
    

    BufRows       dw   ?          ;buffer height (rows)
    BufCols       dw   ?          ;buffer width (columns)
    BufChars      dd   ?          ;number of chars in the buffer


    PtrFileName   DWORD ?   

    hFile     HANDLE ? 
    cci       CONSOLE_CURSOR_INFO <>        


    wHnd      HANDLE ? 
    rHnd      HANDLE ?

    windowSize SMALL_RECT <>
    bufferSize COORD      <>
    startPoint COORD      <>

    howmany dd ?
    buffer  INPUT_RECORD <>

    OSVERSIONINFOEX_NEW STRUCT
      dwOSVersionInfoSize   DWORD ?
      dwMajorVersion        DWORD ?
      dwMinorVersion        DWORD ?
      dwBuildNumber         DWORD ?
      dwPlatformId          DWORD ?
      szCSDVersion          BYTE  128  dup (?)
      wServicePackMajor     WORD  ?
      wServicePackMinor     WORD  ?
      wSuiteMask            WORD  ?
      wProductType          BYTE  ?
      wReserved             BYTE  ?
    OSVERSIONINFOEX_NEW ENDS

    CTEXT MACRO text:VARARG
      LOCAL TxtName
      .DATA
        TxtName BYTE text,0
      .CODE
      EXITM <ADDR TxtName>
    ENDM

    szOS   BYTE   256 dup (?)

    CpuBuffer BYTE 260 dup (?)
    pbuf   DWORD ?



.code



; -------------------------------------------------------------------------
; The data read from the file is converted into CHAR_INFO style. 
; ConsoleBuffer is filled with the content of FileBuffer, padding a zero
; after each byte making it Windows Console API displayable. 
; -------------------------------------------------------------------------

ConvertArea PROC

    lea  esi, FileBuffer
    lea  edi, ConsoleScreen
    
    mov  ecx, CharNumber
    mov  eax, CharNumber
    pxor mm7,mm7
    and ecx,1
    shr eax,1
    jz @no2
    @@:
    movd mm0,dword ptr [esi]
    punpcklbw mm0,mm7
    add esi,4
    movntq qword ptr [edi],mm0
    add edi,8
    dec eax
    jnz @B
    @no2:
    jecxz @done
    movzx eax,word ptr [esi]
    mov [edi],al
    mov [edi+2],ah   
        
    @done:
    emms
    ret 
    
ConvertArea ENDP

; --------------------------------------------------------------------------------
;  The buffer to be displayed is initially zeroed
; -------------------------------------------------------------------------   

ClearBuffer PROC AddrBuffer:DWORD


    mov edx, AddrBuffer
    lea ecx, [edx + MaxSize]
    pxor mm0,mm0
  @@:
    movntq [edx], mm0
    add edx, 8
    cmp edx, ecx
    jl @B
    emms

    ret

ClearBuffer ENDP


;#############################################################################
; Instructions detection code by Alex aka Antariy
OPTION PROLOGUE:NONE
OPTION EPILOGUE:NONE

AxCPUid_Features	struct DWORD
	Is486capable	dd	?
	IsP1		dd	?
	IsP1MMX		dd	?
	IsPPro		dd	?
	IsSSE1		dd	?
	IsSSE2		dd	?
	IsSSE3		dd	?
	IsSSSE3		dd	?
	IsSSE41		dd	?
	IsSSE42		dd	?
	BrandName	db	64 dup (?)
AxCPUid_Features	ends

AxCPUidCodeSizeStart EQU $


; Return values:
; Not zero - if entire structure was filled
; 0 - if CPUID is not supported or not entire struct beed filled.
;     For clearness, check the Is486capable - if 0 then CPUID is not supported,
;     otherwise CPU family less than PIV, but structure are filled properly.

align 16
AxCPUid_FillStructure	proc	lpAxCPUid_Features:PTR AxCPUid_Features
	.data?
		ifrunnedAndSupportCPUid	dd	?
	.code
	
	push ebp
	push ebx
	
	xor eax,eax
	mov ecx,sizeof AxCPUid_Features

	mov ebp,[esp+4+8]
	
	@@:
	mov [ebp+ecx-4].AxCPUid_Features.Is486capable,eax
	add ecx,-4
	jnz @B

	mov ebx,ifrunnedAndSupportCPUid
	dec ebx
	jz @done
	dec ebx
	jz @is486capable
	
	inc ifrunnedAndSupportCPUid
	
	pushfd
	pop ecx
	mov ebx,ecx
	xor ebx,200000h	; switch ID flag
	push ebx
	popfd		; change EFLAGS
	pushfd
	pop ebx
	xor ebx,ecx	; if previous ID bit was equal to current
	test ebx,200000h; then it would be dropped
	jz @done	; otherwise - it set
	
	inc ifrunnedAndSupportCPUid
	
	@is486capable:

	or [ebp].AxCPUid_Features.Is486capable,1
	
	xor eax,eax
	cpuid
	test eax,eax
	jz @done
	
	mov eax,1
	cpuid
	
	shr edx,5	; has RDTSC - so, this is PI at least
	adc [ebp].AxCPUid_Features.IsP1,0
	
	shr edx,2	; has PAE - so, PPro at least
	adc [ebp].AxCPUid_Features.IsPPro,0
	
	shr edx,17	; has MMX
	adc [ebp].AxCPUid_Features.IsP1MMX,0
	
	shr edx,2	; has SSE1
	adc [ebp].AxCPUid_Features.IsSSE1,0
	
	shr edx,1	; has SSE2
	adc [ebp].AxCPUid_Features.IsSSE2,0
	
	
	
	shr ecx,1	; has SSE3
	adc [ebp].AxCPUid_Features.IsSSE3,0
	
	shr ecx,9	; has SSSE3
	adc [ebp].AxCPUid_Features.IsSSSE3,0
	
	shr ecx,10	; has SSE4.1
	adc [ebp].AxCPUid_Features.IsSSE41,0
	
	shr ecx,1	; has SSE4.2
	adc [ebp].AxCPUid_Features.IsSSE42,0
	
	
	; get CPU brand name, if exist
	

	; fix for PIII, not SSE2 capable CPU cannot have brand name
	cmp [ebp].AxCPUid_Features.IsSSE2,0
	jz @done

	mov eax,80000000h
	cpuid
	add eax,eax	; check for zero and no any extended functions
	jz @done
	
	lea ebp,[ebp].AxCPUid_Features.BrandName
	cmp eax,8	; needed at least 80000004h function
	mov eax,0
	jb @done
	
	push esi
	mov esi,80000002h
	@@:
	mov eax,esi
	cpuid
	inc esi
	mov [ebp],eax
	mov [ebp+4],ebx
	mov [ebp+8],ecx
	mov [ebp+12],edx
	add ebp,16
	cmp esi,80000005h
	jb @B		
	pop esi
	
	or eax,1	; in case of terminated brand string
	
	@done:
	pop ebx
	pop ebp
	ret 4
AxCPUid_FillStructure	endp


; Return values:
; 	low order WORD of EAX:
; 		instructions supported:
; 			0 - need upgrade
; 			above 0 - then supported:
; 			1 - MMX
; 			2 - SSE1
; 			3 - SSE2
;			4 - SSE3
; 			5 - SSSE3
; 			6 - SSE4.1
; 			7 - SSE4.2
;	high order WORD of EAX:
;		size of the output string in chars, without termination null
align 16
AxCPUid_Print_ForFrank_IntoBuffer	proc lpStrOut:DWORD
	push ebx
	push esi
	push edi
	
	invoke GetStdHandle,STD_OUTPUT_HANDLE
	xchg eax,ebx
	
	mov esi,[esp+12+4]
		
	invoke AxCPUid_FillStructure,esi
	cmp [esi].AxCPUid_Features.Is486capable,0
	jnz @F	
	
	push eax
	mov edx,esp
	
	push 0
	push edx
	push sizeof @@needupgrade
	push offset @@needupgrade
	push ebx
	call WriteFile
	pop ecx
	xor eax,eax
	jmp @done
	
	@@:
	
	mov edi,[esi].AxCPUid_Features.IsP1MMX
	add edi,[esi].AxCPUid_Features.IsSSE1
	add edi,[esi].AxCPUid_Features.IsSSE2
	add edi,[esi].AxCPUid_Features.IsSSE3
	add edi,[esi].AxCPUid_Features.IsSSSE3
	add edi,[esi].AxCPUid_Features.IsSSE41
	add edi,[esi].AxCPUid_Features.IsSSE42
		
	mov eax,[esi].AxCPUid_Features.IsSSE42
	lea eax,[eax*2+offset @@sse42]

	mov edx,[esi].AxCPUid_Features.IsSSE41
	lea edx,[edx*2+offset @@sse41]
	
	mov ecx,[esi].AxCPUid_Features.IsSSSE3
	lea ecx,[ecx*2+offset @@ssse3]
	
	push eax
	push edx
	push ecx
	
	mov eax,[esi].AxCPUid_Features.IsSSE3
	lea eax,[eax*2+offset @@sse3]
	
	mov edx,[esi].AxCPUid_Features.IsSSE2
	lea edx,[edx*2+offset @@sse2]
	
	mov ecx,[esi].AxCPUid_Features.IsSSE1
	lea ecx,[ecx*2+offset @@sse1]
	
	push eax
	push edx
	push ecx	
	
	mov eax,[esi].AxCPUid_Features.IsP1MMX
	lea eax,[eax*2+offset @@mmx]
	
	lea edx,[esi].AxCPUid_Features.BrandName
	cmp dword ptr [edx],0
	jnz @hasbrandname

	mov ecx,[esi].AxCPUid_Features.Is486capable
	add ecx,[esi].AxCPUid_Features.IsP1
	add ecx,[esi].AxCPUid_Features.IsP1MMX
	add ecx,[esi].AxCPUid_Features.IsPPro
	
	cmp ecx,4
	jb @F
	add ecx,[esi].AxCPUid_Features.IsP1MMX	; PII is PPro with MMX
	add ecx,[esi].AxCPUid_Features.IsSSE1	; PIII is PPro with MMX and SSE1
	@@:
		
	mov edx,[ecx*4+offset @@earlycpus]
		
	@hasbrandname:
	add edx,1
	cmp byte ptr [edx-1]," "
	jz @hasbrandname
	
	dec edx
		
	push eax
	push edx
	push offset @@fmtFeatures
	push esi
	call wsprintf
	
	shl eax,16	; put count of chars into high word

	add esp,10*4

	add eax,edi	; put instructions set supported into AX
	@done:
	pop edi
	pop esi
	pop ebx
	ret 4
	
	even
	@@needupgrade	db	"This is time for upgrade indeed, i386 or early i486 :)"
	even
	@@i486		db	"Old-good 80486",0
	even
	@@p1		db	"Pentium",0
	even
	@@pmmx		db	"Pentium with MMX Technology",0
	even
	@@ppro		db	"Pentium Pro",0
	even
	@@p2		db	"Pentium II",0
	even
	@@p3		db	"Pentium III",0
	
	align 4
	@@earlycpus	dd	0
			dd	offset @@i486
			dd	offset @@p1
			dd	offset @@pmmx
			dd	offset @@ppro
			dd	offset @@p2
			dd	offset @@p3
	
	even
	@@fmtFeatures	db	"%s with %s%s%s%s%s%s%s",0
	even
	@@mmx		db	0,0
			db	"MMX",0
	even
	@@sse1		db	0,0
			db	", SSE1",0
	even
	@@sse2		db	0,0
			db	", SSE2",0
	even
	@@sse3		db	0,0
			db	", SSE3",0
	even
	@@ssse3		db	0,0
			db	", SSSE3",0
	even
	@@sse41		db	0,0
			db	", SSE4.1",0
	even
	@@sse42		db	0,0
			db	", SSE4.2",0
	
	
AxCPUid_Print_ForFrank_IntoBuffer	endp

OPTION PROLOGUE:PrologueDef
OPTION EPILOGUE:EpilogueDef
;#############################################################################









start: 

; ---------------------------------------------------------------------------
; The Main Proc shows the program flow.
; ---------------------------------------------------------------------------

Main PROC

    CALL   ConsoleSize 

    ;INVOKE ClearBuffer, ADDR ConsoleScreen ; since repacking proc, this is not needed

    CALL   GetConsole

    CALL   InitProc

    CALL   ConvertArea

    INVOKE DisplayFmt, offset ConsoleScreen

    mov    ActualColor, Zero
    
    INVOKE GetWindowsOS, ADDR szOS
    
    INVOKE lstrcat, ADDR szOS, CTEXT(13,10)

    INVOKE SetColorTo, BrightYellow, DarkBlue

    mov    Lenght, len(ADDR szOS)
   
    INVOKE DisplayAt, 7, 1, ADDR szOS, DWORD PTR Lenght


	;#################################################################
	;# This is addeded    
    invoke AxCPUid_Print_ForFrank_IntoBuffer,offset CpuBuffer
    ; NOTE: After a call to ^^^ this proc, the EAX is constructed, read
    ; comments above proc
        
    movzx ebx,ax	; maximal instruction sets, supported by CPU
    ; for example, if test include code, which is required SSSE3, then need to check
    ; an AX (or EBX in this sample) for 5. If number is smaller than 5, then SSSE3 is not supported
    ; and needed to manage that thing.
    
    shr eax,16

    push eax	; prepushed value of Length for DisplayAt proc

    invoke szMonoSpace,offset CpuBuffer

    push eax	; buffer with CPU string
    
    push 2	; y
    push 7	; x
    call DisplayAt
    	;#################################################################


    mov    Lenght, len(ADDR AlgoToTest)
   
    INVOKE DisplayAt, 7, 4, ADDR AlgoToTest, DWORD PTR Lenght

      
    CALL   AnyKey

    CALL   ShowTheCursor

Finish: 

    exit

Main ENDP

 
; ---------------------------------------------------------------------------
; The size, title and buffer of the Console are set.
; Max Console Size is set to 34 Rows and 90 Cols.
; ---------------------------------------------------------------------------

ConsoleSize PROC

    INVOKE AllocConsole

    INVOKE GetStdHandle, STD_INPUT_HANDLE
    mov rHnd,eax

    INVOKE GetStdHandle, STD_OUTPUT_HANDLE
    mov wHnd,eax 

    INVOKE SetConsoleTitle, ADDR ConTitle
  
    CALL   HideTheCursor

    mov windowSize.Left, Zero
    mov windowSize.Right, (MaxCols -1)
    mov windowSize.Top, Zero
    mov windowSize.Bottom, (MaxRows -1)

    
    mov bufferSize.x, MaxCols
    mov bufferSize.y, MaxRows

    INVOKE SetConsoleWindowInfo, wHnd, TRUE, ADDR windowSize

    INVOKE SetConsoleScreenBufferSize, wHnd, DWORD PTR bufferSize

    ret

ConsoleSize ENDP

; ---------------------------------------------------------------------------
; Reads the Screen File Corner Positions.
; ---------------------------------------------------------------------------
;    The initial 4 bytes contain:
;
;    RowInitial, RowFinal, ColInitial, ColFinal of the Screen File to display
; ---------------------------------------------------------------------------
 

GetConsole PROC 


    movzx eax, RowFinal
    mov   R2,  ax
    inc   eax
    sub   al,  RowInitial

    mov   BufRows, ax

    movzx ebx, ColFinal
    mov   C2,  bx    
    inc   ebx
    sub   bl,  ColInitial

    mov   BufCols, bx

    movzx ax, RowInitial  
    mov   R1, ax

    dec   R1
    dec   R2
    movzx ax, ColInitial
    mov   C1, ax 
    dec   C1   
    dec   C2

    mov   ebx, CharNumber
    shr   ebx, 1		;!!! Why is this?
    mov   BufChars, ebx

    ret 
    
GetConsole ENDP

;----------------------------------------------------------------------------
; The Cursor is made visible
;----------------------------------------------------------------------------

ShowTheCursor PROC

    INVOKE GetConsoleCursorInfo,wHnd,addr cci
    mov cci.bVisible,TRUE
    INVOKE SetConsoleCursorInfo,wHnd,addr cci
    ret
    
ShowTheCursor ENDP

;----------------------------------------------------------------------------
; The Cursor is made invisible
;----------------------------------------------------------------------------
    


HideTheCursor PROC

    INVOKE GetConsoleCursorInfo,wHnd,addr cci
    mov cci.bVisible,FALSE
    INVOKE SetConsoleCursorInfo,wHnd,addr cci
    ret
    
HideTheCursor ENDP

;----------------------------------------------------------------------------
; After the Screen File has been read, the parameters to display it are set
;----------------------------------------------------------------------------

InitProc PROC

 
    mov ax, C1
    mov windowSize.Left, ax
    mov ax, C2
    mov windowSize.Right, ax
    mov ax, R1
    mov windowSize.Top, ax
    mov ax, R2
    mov windowSize.Bottom, ax
   
    mov ax, BufRows
    mov bx, BufCols
    
    mov bufferSize.x, bx
    mov bufferSize.y, ax

    mov startPoint.x, Zero
    mov startPoint.y, Zero


    ret

InitProc ENDP

;----------------------------------------------------------------------------
; The Screen File is displayed on the Console 
;----------------------------------------------------------------------------

DisplayFmt PROC szFmtName:DWORD 

    INVOKE WriteConsoleOutput, wHnd, szFmtName, DWORD PTR bufferSize, 
           DWORD PTR startPoint, offset windowSize 

    ret

DisplayFmt ENDP

;----------------------------------------------------------------------------
; The program waits for a key pressed
;----------------------------------------------------------------------------
; Returns: key code in buffer.KeyEvent.wVirtualKeyCode WORD size 
; -------------------------------------------------------------------------

AnyKey PROC

again:  

    INVOKE ReadConsoleInput,rHnd,offset buffer,1,offset howmany
    cmp buffer.EventType,KEY_EVENT
    jnz again

    cmp buffer.KeyEvent.bKeyDown,0
    jz again

    mov ax, buffer.KeyEvent.wVirtualKeyCode
    cmp al, EscKey
    jz  ClosePgm

    cmp al, R_Key
    jz  R_Key_Press

    cmp al, r_Key
    jz  R_Key_Press

    cmp al, S_Key
    jz  S_Key_Press

    cmp al, s_Key
    jz  S_Key_Press

    cmp al, V_Key
    jz  V_Key_Press

    cmp al, v_Key
    jz  V_Key_Press

    jmp again

R_Key_Press:

    mov    Lenght, len(ADDR R_Key_Pressed)
   
    INVOKE DisplayAt, 4, 8, ADDR R_Key_Pressed, DWORD PTR Lenght 

    jmp again

S_Key_Press:

    mov    Lenght, len(ADDR S_Key_Pressed)
   
    INVOKE DisplayAt, 4, 8, ADDR S_Key_Pressed, DWORD PTR Lenght 

    jmp again

V_Key_Press:

    mov    Lenght, len(ADDR V_Key_Pressed)
   
    INVOKE DisplayAt, 4, 8, ADDR V_Key_Pressed, DWORD PTR Lenght 

    jmp again

ClosePgm:

    ret 

AnyKey ENDP


; --------------------------------------------------------------------------------
; Check for Windows Version - GregL updated version to detect Win 7
; -------------------------------------------------------------------------

GetWindowsOS PROC pszOS:PTR BYTE

    LOCAL hMod:DWORD
    LOCAL dwType:DWORD
    LOCAL pGNSI:PTR
    LOCAL pGPI:PTR
    LOCAL osvi:OSVERSIONINFOEX_NEW
    LOCAL sysinfo:SYSTEM_INFO
    LOCAL szBuild[16]:BYTE
    LOCAL szBuf[128]:BYTE
    
    INVOKE RtlZeroMemory, ADDR osvi, SIZEOF OSVERSIONINFOEX
    INVOKE RtlZeroMemory, ADDR sysinfo, SIZEOF SYSTEM_INFO
    mov osvi.dwOSVersionInfoSize, SIZEOF OSVERSIONINFOEX
    INVOKE GetVersionEx, ADDR osvi
    ; Call GetNativeSystemInfo if supported or GetSystemInfo otherwise.
    INVOKE GetModuleHandle, CTEXT("kernel32.dll")
    mov hMod, eax
    INVOKE GetProcAddress, hMod, CTEXT("GetNativeSystemInfo")
    mov pGNSI, eax
    .IF pGNSI != NULL
        lea eax, sysinfo
        push eax
        call pGNSI
    .ELSE
        INVOKE GetSystemInfo, ADDR sysinfo
    .ENDIF  
    
    .IF (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT) && (osvi.dwMajorVersion > 3)
        ; This program does not support dwMajorVersion less than 4
        INVOKE lstrcpy, pszOS, CTEXT("Microsoft ")
        .IF (osvi.dwMajorVersion == 6)
            .IF (osvi.dwMinorVersion == 0)
                ; Windows Vista or Windows Server 2008
                .IF osvi.wProductType == VER_NT_WORKSTATION
                    INVOKE lstrcat, pszOS, CTEXT("Windows Vista ")
                .ELSE
                    INVOKE lstrcat, pszOS, CTEXT("Windows Server 2008")
                .ENDIF
            .ELSEIF (osvi.dwMinorVersion == 1)
                ; Windows 7 or Windows Server 2008 R2
                .IF osvi.wProductType == VER_NT_WORKSTATION
                    INVOKE lstrcat, pszOS, CTEXT("Windows 7 ")
                .ELSE
                    INVOKE lstrcat, pszOS, CTEXT("Windows Server 2008 R2")
                .ENDIF
            .ENDIF
            
            INVOKE GetProcAddress, hMod, CTEXT("GetProductInfo")
            mov pGPI, eax
            lea eax, dwType
            push eax
            push 0
            push 0
            push osvi.dwMinorVersion
            push osvi.dwMajorVersion
            call pGPI
            .IF dwType == PRODUCT_ULTIMATE
               INVOKE lstrcat, pszOS, CTEXT("Ultimate Edition")
            .ELSEIF dwType == PRODUCT_PROFESSIONAL
               INVOKE lstrcat, pszOS, CTEXT("Professional")               
            .ELSEIF dwType == PRODUCT_HOME_PREMIUM
               INVOKE lstrcat, pszOS, CTEXT("Home Premium Edition")
            .ELSEIF dwType == PRODUCT_HOME_BASIC
               INVOKE lstrcat, pszOS, CTEXT("Home Basic Edition")
            .ELSEIF dwType == PRODUCT_ENTERPRISE
               INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition")
            .ELSEIF dwType == PRODUCT_BUSINESS
               INVOKE lstrcat, pszOS, CTEXT("Business Edition")
            .ELSEIF dwType == PRODUCT_STARTER
               INVOKE lstrcat, pszOS, CTEXT("Starter Edition")
            .ELSEIF dwType == PRODUCT_CLUSTER_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Cluster Server Edition")
            .ELSEIF dwType == PRODUCT_DATACENTER_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Datacenter Edition")
            .ELSEIF dwType == PRODUCT_DATACENTER_SERVER_CORE
               INVOKE lstrcat, pszOS, CTEXT("Datacenter Edition (core installation)")
            .ELSEIF dwType == PRODUCT_ENTERPRISE_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition")
            .ELSEIF dwType == PRODUCT_ENTERPRISE_SERVER_CORE
               INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition (core installation)")
            .ELSEIF dwType == PRODUCT_ENTERPRISE_SERVER_IA64
               INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition for Itanium-based Systems")
            .ELSEIF dwType == PRODUCT_SMALLBUSINESS_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Small Business Server")
            .ELSEIF dwType == PRODUCT_SMALLBUSINESS_SERVER_PREMIUM
               INVOKE lstrcat, pszOS, CTEXT("Small Business Server Premium Edition")
            .ELSEIF dwType == PRODUCT_STANDARD_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Standard Edition")
            .ELSEIF dwType == PRODUCT_STANDARD_SERVER_CORE
               INVOKE lstrcat, pszOS, CTEXT("Standard Edition (core installation)")
            .ELSEIF dwType == PRODUCT_WEB_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Web Server Edition")
            .ENDIF    
            .IF sysinfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64
                INVOKE lstrcat, pszOS, CTEXT( ", 64-bit" )
            .ELSEIF sysinfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_INTEL
                INVOKE lstrcat, pszOS, CTEXT(", 32-bit")
            .ENDIF
        .ELSEIF (osvi.dwMajorVersion == 5) && (osvi.dwMinorVersion == 2)
            ; Windows Server 2003    
            INVOKE GetSystemMetrics, SM_SERVERR2
            .IF eax
                INVOKE lstrcat, pszOS, CTEXT("Windows Server 2003 R2, ")
            .ELSEIF osvi.wSuiteMask == VER_SUITE_STORAGE_SERVER    
                INVOKE lstrcat, pszOS, CTEXT( "Windows Storage Server 2003")
            .ELSEIF (osvi.wProductType == VER_NT_WORKSTATION) && (sysinfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64)
                INVOKE lstrcat, pszOS, CTEXT( "Windows XP Professional x64 Edition")
            .ELSE
                INVOKE lstrcat, pszOS, CTEXT("Windows Server 2003, ")
            .ENDIF
            .IF osvi.wProductType != VER_NT_WORKSTATION
                .IF sysinfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_IA64
                    .IF osvi.wSuiteMask & VER_SUITE_DATACENTER
                       INVOKE lstrcat, pszOS, CTEXT("Datacenter Edition for Itanium-based Systems")
                    .ELSEIF osvi.wSuiteMask & VER_SUITE_ENTERPRISE
                       INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition for Itanium-based Systems")
                    .ENDIF   
                .ENDIF
            .ELSEIF sysinfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64
                .IF osvi.wSuiteMask & VER_SUITE_DATACENTER
                    INVOKE lstrcat, pszOS, CTEXT("Datacenter x64 Edition")
                .ELSEIF osvi.wSuiteMask & VER_SUITE_ENTERPRISE
                    INVOKE lstrcat, pszOS, CTEXT("Enterprise x64 Edition")
                .ELSE 
                    INVOKE lstrcat, pszOS, CTEXT("Standard x64 Edition")
                .ENDIF    
            .ELSE
                .IF osvi.wSuiteMask & VER_SUITE_COMPUTE_SERVER
                    INVOKE lstrcat, pszOS, CTEXT("Compute Cluster Edition")
                .ELSEIF osvi.wSuiteMask & VER_SUITE_DATACENTER
                    INVOKE lstrcat, pszOS, CTEXT("Datacenter Edition")
                .ELSEIF osvi.wSuiteMask & VER_SUITE_ENTERPRISE
                    INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition")
                .ELSEIF osvi.wSuiteMask & VER_SUITE_BLADE
                    INVOKE lstrcat, pszOS, CTEXT("Web Edition")
                .ELSE 
                    INVOKE lstrcat, pszOS, CTEXT("Standard Edition")
                .ENDIF    
            .ENDIF
        .ELSEIF (osvi.dwMajorVersion == 5) && (osvi.dwMinorVersion == 1)
            ; Windows XP            
            INVOKE lstrcat, pszOS, CTEXT("Windows XP ")
            .IF osvi.wSuiteMask & VER_SUITE_PERSONAL
                INVOKE lstrcat, pszOS, CTEXT("Home Edition")
            .ELSE 
                INVOKE lstrcat, pszOS, CTEXT("Professional")
            .ENDIF    
        .ELSEIF (osvi.dwMajorVersion == 5) && (osvi.dwMinorVersion == 0)
            ; Windows 2000
            INVOKE lstrcat, pszOS, CTEXT("Windows 2000 ")
             .IF osvi.wProductType == VER_NT_WORKSTATION
                 INVOKE lstrcat, pszOS, CTEXT("Professional" )
             .ELSE
                 .IF osvi.wSuiteMask & VER_SUITE_DATACENTER
                     INVOKE lstrcat, pszOS, CTEXT("Datacenter Server" )
                 .ELSEIF osvi.wSuiteMask & VER_SUITE_ENTERPRISE
                     INVOKE lstrcat, pszOS, CTEXT("Advanced Server")
                 .ELSE 
                     INVOKE lstrcat, pszOS, CTEXT("Server")
                 .ENDIF  
             .ENDIF    
        .ELSEIF osvi.dwMajorVersion == 4
            ; Windows NT 4.0
            INVOKE lstrcat, pszOS, CTEXT("Windows NT 4.0")
        .ENDIF  
        ; Include service pack (if any) and build number.
        INVOKE lstrlen, ADDR osvi.szCSDVersion 
        .IF eax > 0
            INVOKE lstrcat, pszOS, CTEXT(" ")
            INVOKE lstrcat, pszOS, ADDR osvi.szCSDVersion
        .ENDIF
        INVOKE wsprintf, ADDR szBuild, CTEXT(" (build %d)"), osvi.dwBuildNumber
        INVOKE lstrcat, pszOS, ADDR szBuild
        xor eax, eax
    .ELSE
        INVOKE lstrcpy, pszOS, CTEXT("This program does not support Windows versions less than Windows NT 4.0")
        mov eax, 1
    .ENDIF
    ret
GetWindowsOS ENDP

; --------------------------------------------------------------------------------
; Set the attribute colors for next PRINT/DISPLAY instruction
;----------------------------------------------------------------------------


SetColorTo PROC Fore:BYTE, Back:BYTE

    xor  eax, eax

    mov  al, Back
    shl  al, 4
    add  al, Fore
    mov  ChosenColor, ax 

    invoke SetConsoleTextAttribute, wHnd, ChosenColor

    ret

SetColorTo ENDP

; --------------------------------------------------------------------------------
; A string is displayed at Column and Row for the passed Lenght
; ----------------------------------------------------------------------------

DisplayAt PROC x:DWORD, y:DWORD, StringBuffer:DWORD, lenght:DWORD
   
    LOCAL n_of_chars:DWORD
    LOCAL position:COORD
   
    mov eax, x
    mov position.x, ax
    mov eax, y
    mov position.y, ax
   
    invoke SetConsoleCursorPosition, wHnd, DWORD PTR position
    invoke WriteConsole, wHnd, StringBuffer, lenght, ADDR n_of_chars, NULL
   
    ret
    
DisplayAt ENDP


; ----------------------------------------------------------------------------
; --------------------------------------------------------------------------------

end start
